@@ -30,19 +30,21 @@ DIR_ONTV = "/apps/ontv"
 
 KEY_GRABBER_COMMAND             = "/general/grabber_command"
 KEY_XMLTV_FILE                  = "/general/xmltv_file"
-KEY_DISPLAY_CURRENT_PROGRAMS    = "/general/display_current_programs"
-KEY_DISPLAY_UPCOMING_PROGRAMS   = "/general/display_upcoming_programs"
 KEY_ENABLE_HOTKEYS              = "/general/enable_hotkeys"
 KEY_SHOW_WINDOW_HOTKEY          = "/general/show_window_hotkey"
 KEY_SHOW_SEARCH_PROGRAM_HOTKEY  = "/general/show_search_program_hotkey"
+KEY_DISPLAY_CURRENT_PROGRAMS    = "/general/display_current_programs"
+KEY_DISPLAY_UPCOMING_PROGRAMS   = "/general/display_upcoming_programs"
+KEY_UPCOMING_PROGRAMS_BELOW     = "/general/upcoming_programs_below"
 
 FUNCTION_SUFFIXES = {KEY_GRABBER_COMMAND:             'string',
                      KEY_XMLTV_FILE:                  'string',
-                     KEY_DISPLAY_CURRENT_PROGRAMS:    'bool',
-                     KEY_DISPLAY_UPCOMING_PROGRAMS:   'bool',
                      KEY_ENABLE_HOTKEYS:              'bool',
                      KEY_SHOW_WINDOW_HOTKEY:          'string',
-                     KEY_SHOW_SEARCH_PROGRAM_HOTKEY:  'string'}
+                     KEY_SHOW_SEARCH_PROGRAM_HOTKEY:  'string',
+                     KEY_DISPLAY_CURRENT_PROGRAMS:    'bool',
+                     KEY_DISPLAY_UPCOMING_PROGRAMS:   'bool',
+                     KEY_UPCOMING_PROGRAMS_BELOW:     'bool'}
 
 class Configuration(object):
     """Singleton representing the configuration"""
@@ -188,4 +190,17 @@ class Configuration(object):
     display_upcoming_programs = property(get_display_upcoming_programs,
                                          set_display_upcoming_programs)
 
+    def get_upcoming_programs_below(self, use_cache=True):
+        if use_cache:
+            return self.option_cache[KEY_UPCOMING_PROGRAMS_BELOW]
+        else:
+            return self.__get_option(KEY_UPCOMING_PROGRAMS_BELOW)
+
+    def set_upcoming_programs_below(self, upcoming_programs_below):
+        self.__set_option(KEY_UPCOMING_PROGRAMS_BELOW,
+                          upcoming_programs_below)
+
+    upcoming_programs_below = property(get_upcoming_programs_below,
+                                       set_upcoming_programs_below)
+
 # vim: set sw=4 et sts=4 tw=79 fo+=l:
