@@ -24,6 +24,7 @@ import gtk
 import gobject
 
 import GUI
+from GUI import ProgramContextMenu
 from ProgramDialog import ProgramDialog
 from Reminders import Reminder
 from ontv import NAME
@@ -80,17 +81,6 @@ class SearchDialog:
         program_column.set_cell_data_func(crtext, self.__crtext_cell_data_func)
         self.search_treeview.append_column(program_column)
 
-        self.search_treeview_menu = gtk.Menu()
-        self.details_imi = gtk.ImageMenuItem(gtk.STOCK_INFO)
-        self.details_imi.child.set_markup_with_mnemonic(_("_Details"))
-        self.details_imi.show()
-        self.search_treeview_menu.append(self.details_imi)
-
-        self.add_reminder_imi = gtk.ImageMenuItem("gnome-stock-timer")
-        self.add_reminder_imi.child.set_markup_with_mnemonic(_("_Add reminder"))
-        self.add_reminder_imi.show()
-        self.search_treeview_menu.append(self.add_reminder_imi)
-
     def __crt_cell_data_func(self, column, cell, model, iter):
         program = model.get_value(iter, 0)
         markup = "<b>%s-%s</b>\n<i>%s</i>"
@@ -112,12 +102,6 @@ class SearchDialog:
         selection.connect("changed", self.__search_treeview_selection_changed)
         self.search_treeview.connect("row-activated",
                                      self.__search_treeview_row_activated)
-        self.search_treeview.connect("button-press-event",
-                                     self.__search_treeview_button_press,
-                                     self.search_treeview_menu)
-        self.details_imi.connect("activate", self.__details_imi_activate)
-        self.add_reminder_imi.connect("activate",
-                                      self.__add_reminder_imi_activate)
 
         self.close_button.connect("clicked", self.__close_button_clicked)
         self.details_button.connect("clicked", self.__details_button_clicked)
@@ -168,12 +152,12 @@ class SearchDialog:
             reminder = Reminder(program.title,
                                 program.channel.markup_escaped_name)
             if not reminder in self.reminders.reminders:
-                self.add_reminder_imi.set_sensitive(True)
+                self.search_treeview_menu.add_reminder_imi.set_sensitive(True)
             else:
-                self.add_reminder_imi.set_sensitive(False)
+                self.search_treeview_menu.add_reminder_imi.set_sensitive(False)
         else:
             self.details_button.set_sensitive(False)
-            self.add_reminder_imi.set_sensitive(False)
+            self.search_treeview_menu.add_reminder_imi.set_sensitive(False)
 
     def __search_treeview_row_activated(self, treeview, path, column):
         self.details_button.emit("clicked")
@@ -184,24 +168,12 @@ class SearchDialog:
             path = treeview.get_path_at_pos(int(event.x), int(event.y))[0]
             program = search_model[path][0]
             if program.description and program.description != "":
-                self.details_imi.set_sensitive(True)
+                self.search_treeview_menu.details_imi.set_sensitive(True)
             else:
-                self.details_imi.set_sensitive(False)
+                self.search_treeview_menu.details_imi.set_sensitive(False)
 
             menu.popup(None, None, None, event.button, event.time)
 
-    def __details_imi_activate(self, menuitem):
-        program = GUI.get_selected_value(self.search_treeview)
-        self.__show_program_dialog(program)
-
-    def __add_reminder_imi_activate(self, menuitem):
-        program = GUI.get_selected_value(self.search_treeview)
-        reminder = Reminder(program.title, program.channel.markup_escaped_name)
-        if self.reminders.add(reminder):
-            self.reminders_model.append([reminder])
-            self.reminders.save()
-            self.add_reminder_imi.set_sensitive(False)
-
     def __close_button_clicked(self, button):
         self.dialog.hide()
 
@@ -227,6 +199,17 @@ class SearchDialog:
                 break
             channels_iter = channels_model.iter_next(channels_iter)
 
+    def create_search_treeview_menu(self, reminders_model):
+        self.search_treeview_menu = ProgramContextMenu(self.get_program,
+                                                       self.reminders,
+                                                       reminders_model)
+        self.search_treeview.connect("button-press-event",
+                                     self.__search_treeview_button_press,
+                                     self.search_treeview_menu)
+
+    def get_program(self):
+        return GUI.get_selected_value(self.search_treeview)
+
     def show(self, uicomponent=None, verb=None):
         self.present()
 
